from unittest2 import TestCase
from ftw.inflator.creation.sections import helpers


class TestMapRecursiveValue(TestCase):

    def test_simple_list(self):
        data = ['foo', 'upper::bar', 'baz']
        result = helpers.map_recursive(
            lambda item: item.startswith('upper::'),
            lambda item: item.lstrip('upper::').upper(),
            data)

        self.assertEquals(result, ['foo', 'BAR', 'baz'])

    def test_simple_tuple(self):
        data = ('foo', 'upper::bar', 'baz')
        result = helpers.map_recursive(
            lambda item: item.startswith('upper::'),
            lambda item: item.lstrip('upper::').upper(),
            data)

        self.assertEquals(result, ('foo', 'BAR', 'baz'))

    def test_simple_dict(self):
        data = {'foo': 'upper::bar',
                'upper::baz': 5,
                'bar': 'bar'}

        result = helpers.map_recursive(
            lambda item: item.startswith('upper::'),
            lambda item: item.lstrip('upper::').upper(),
            data)

        self.assertEquals(result, {
                'foo': 'BAR',
                'BAZ': 5,
                'bar': 'bar'})

    def test_nested(self):
        data = ({'foo': ['bar', {'baz': 'upper::foo',
                                 'upper::foo': 2}]},
                3)

        result = helpers.map_recursive(
            lambda item: item.startswith('upper::'),
            lambda item: item.lstrip('upper::').upper(),
            data)

        self.assertEquals(result, (
                {'foo': ['bar', {'baz': 'FOO',
                                 'FOO': 2}]},
                3))



from Products.CMFPlone.browser.admin import AddPloneSite
from Products.CMFPlone.browser.admin import Overview
from ftw.inflator import _
from ftw.inflator.bundle import get_bundle_by_name
from ftw.inflator.bundle import get_bundles
from ftw.inflator.customization import get_merged_customizations
from ftw.inflator.interfaces import EXTENSION_PROFILES


class InflateView(AddPloneSite):

    default_extension_profiles = EXTENSION_PROFILES

    def __call__(self):
        form = self.request.form
        submitted = form.get('form.submitted', False)

        if submitted:
            site_id = form.get('site_id', 'platform')

            bundle = get_bundle_by_name(form.get('bundle'))

            site = bundle.install(
                self.context,
                site_id,
                title=form.get('title', ''),
                language=form.get('default_language', 'en'),
                extension_profiles=self.default_extension_profiles)

            self.request.response.redirect(site.absolute_url())

        return self.index()

    def get_site_id(self):
        id_ = self.request.get('site_id', 'platform')
        while id_ in self.context.objectIds():
            try:
                num = int(id_[-1])
            except ValueError:
                id_ = id_ + str(1)
            else:
                num += 1
                id_ = id_[:-1] + str(num)

        return id_

    def get_customization(self):
        return get_merged_customizations()

    def get_bundle_options(self):
        def option(item):
            index, bundle = item
            return {
                'name': bundle.title.encode('utf-8'),
                'title': bundle.title,
                'description': bundle.description,
                'index': index}

        return map(option, enumerate(get_bundles()))


class InflateOverview(Overview):

    def get_bundle_title(self):
        customization = get_merged_customizations()
        return _(u'Create a new ${product} site',
                 mapping={'product': customization.product})

    def get_customization(self):
        return get_merged_customizations()



from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from zope.interface import classProvides
from zope.interface import implements


class ResolveTree(object):
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous

    def __iter__(self):
        for item in self.previous:
            for node in self._resolve(item):
                yield node

    def _resolve(self, item):
        children = []
        if '_children' in item:
            children = item['_children']
            del item['_children']

        yield item

        for child in children:
            child_path = None
            if '_id' in child:
                child_path = child.pop('_id')
            elif '_path' in child:
                child_path = child.pop('_path').strip('/')

            assert child_path, 'child _id or _path must be specified'
            child['_path'] = '/'.join((item['_path'].rstrip('/'),
                                       child_path))

            for node in self._resolve(child):
                yield node



from Products.CMFCore.utils import getToolByName
from ftw.inflator.testing import INFLATOR_FUNCTIONAL_TESTING
from plone.app.testing import applyProfile
from plone.multilingual.interfaces import ITranslationManager
from unittest2 import TestCase


class TestMultilingualContentCreation(TestCase):

    layer = INFLATOR_FUNCTIONAL_TESTING

    def setUp(self):
        super(TestMultilingualContentCreation, self).setUp()
        self.portal = self.layer['portal']

        wftool = getToolByName(self.portal, 'portal_workflow')
        # Folder is the default language folder type, for which
        # plone.app.multilingual's setup expects a "publish" transition.
        wftool.setChainForPortalTypes(['Folder'],
                                      'simple_publication_workflow')

        applyProfile(self.portal, 'ftw.inflator:setup-language')
        applyProfile(self.portal, 'plone.app.multilingual:default')
        applyProfile(self.portal, 'ftw.inflator.tests:multilingual_creation')

    def test_language_folder_properties_are_set(self):
        obj = self.portal.get('de')
        self.assertEqual(obj.getProperty('layout'), 'there')

    def test_content_creation_from_flat_translated_json(self):
        parent = self.portal.get('en').get('accessibility')
        folder = parent.get('folderwithpath')

        self.assertIsNotNone(folder,
                            'Missing en/accessibility/folderwithpath')
        self.assertEquals('Folder', folder.Title())

        parent = self.portal.get('de').get('barrierefreiheit')
        folder = parent.get('ordnermitpfad')

        self.assertIsNotNone(folder,
                             'Missing de/barrierefreiheit/ordnermitpfad')
        self.assertEquals('Ordner', folder.Title())

    def test_sets_up_language_folders(self):
        self.assertIsNotNone(self.portal.get('en'),
                        'Expected English language folder at /en')

        self.assertIsNotNone(self.portal.get('de'),
                        'Expected German language folder at /de')

    def test_content_is_translated_into_respective_language(self):
        english_content = self.portal.get('en').get('accessibility')
        self.assertIsNotNone(english_content, 'Missing page /en/accessibility')
        self.assertEquals('Accessibility', english_content.Title())

        german_content = self.portal.get('de').get('barrierefreiheit')
        self.assertIsNotNone(german_content,
                            'Missing page /de/barrierefreiheit')
        self.assertEquals('Barrierefreiheit', german_content.Title())

    def test_multilingual_content_is_linked(self):
        english_content = self.portal.get('en').get('accessibility')
        self.assertIsNotNone(english_content, 'Missing page /en/accessibility')

        german_content = self.portal.get('de').get('barrierefreiheit')
        self.assertIsNotNone(german_content,
                             'Missing page /de/barrierefreiheit')

        manager = ITranslationManager(english_content)
        self.assertEquals(german_content,
                          manager.get_translation('de'),
                          'English and German content should be linked.')

    def test_content_creation_import_step_depends_on_languagetool(self):
        # The content creation import step needs to depend on the
        # "languagetool" import step when plone.app.multilingual is installed.
        # The "languagetool" import step is not Plone core.

        setup_tool = getToolByName(self.portal, 'portal_setup')
        import_step_name = 'ftw.inflator.content_creation'
        metadata = setup_tool.getImportStepMetadata(import_step_name)
        self.assertIn('languagetool', metadata.get('dependencies', ()))



from Products.CMFPlone.factory import _DEFAULT_PROFILE
from ftw.inflator.interfaces import IInflatorBundle
from ftw.inflator.testing import META_ZCML
from ftw.testing import MockTestCase
from zope.component import getUtilitiesFor
from zope.configuration.config import ConfigurationConflictError


class TestBundlesZCML(MockTestCase):

    layer = META_ZCML

    def get_utilites(self):
        return list(getUtilitiesFor(IInflatorBundle))

    def test_bundle_zcml_registers_utility(self):
        self.layer.load_zcml_string('\n'.join((
                    '<configure xmlns:inflator="http://namespaces.zope.org/inflator"'
                    '           i18n_domain="my.package">',
                    '    <inflator:bundle',
                    '        title="foo"',
                    '        profiles="plonetheme.sunburst:default" />',
                    '</configure>'
                    )))

        utilites = self.get_utilites()
        self.assertEquals(len(utilites), 1)

        name, obj = utilites[0]
        self.assertEquals(name, u'foo')
        self.assertEquals(obj.title, u'foo')

    def test_title_and_description_translated(self):
        self.layer.load_zcml_string('\n'.join((
                    '<configure xmlns:inflator="http://namespaces.zope.org/inflator"'
                    '           i18n_domain="my.package">',
                    '    <inflator:bundle',
                    '        title="foo"',
                    '        description="bar"'
                    '        profiles="plonetheme.sunburst:default" />',
                    '</configure>'
                    )))

        utilites = self.get_utilites()
        self.assertEquals(len(utilites), 1)
        _name, obj = utilites[0]

        self.assertEquals(obj.title, u'foo')
        self.assertEquals(obj.title.default, None)
        self.assertEquals(type(obj.title).__name__, 'Message')
        self.assertEquals(obj.title.domain, 'my.package')

        self.assertEquals(obj.description, u'bar')
        self.assertEquals(obj.description.default, None)
        self.assertEquals(type(obj.description).__name__, 'Message')
        self.assertEquals(obj.description.domain, 'my.package')

    def test_profiles_is_list(self):
        self.layer.load_zcml_string('\n'.join((
                    '<configure xmlns:inflator="http://namespaces.zope.org/inflator"'
                    '           i18n_domain="my.package">',
                    '    <inflator:bundle',
                    '        title="foo"',
                    '        profiles="plonetheme.sunburst:default    ',
                    '                  Products.CMFPlone:plone-content  my.package:default" />',
                    '</configure>'
                    )))

        utilites = self.get_utilites()
        self.assertEquals(len(utilites), 1)
        _name, obj = utilites[0]

        self.assertEquals(obj.profiles, [u'plonetheme.sunburst:default',
                                         u'Products.CMFPlone:plone-content',
                                         u'my.package:default'])

    def test_defaults(self):
        self.layer.load_zcml_string('\n'.join((
                    '<configure xmlns:inflator="http://namespaces.zope.org/inflator"'
                    '           i18n_domain="my.package">',
                    '    <inflator:bundle',
                    '        title="foo"',
                    '        profiles="plonetheme.sunburst:default" />',
                    '</configure>'
                    )))

        utilites = self.get_utilites()
        self.assertEquals(len(utilites), 1)
        _name, obj = utilites[0]

        self.assertEqual(obj.title, u'foo')
        self.assertEqual(obj.profiles, [u'plonetheme.sunburst:default'])
        self.assertEqual(obj.description, u'')
        self.assertEqual(obj.base, _DEFAULT_PROFILE)
        self.assertEqual(obj.standard, False)

    def test_multiple_bundles(self):
        self.layer.load_zcml_string('\n'.join((
                    '<configure xmlns:inflator="http://namespaces.zope.org/inflator"'
                    '           i18n_domain="my.package">',

                    '    <inflator:bundle',
                    '        title="foo"',
                    '        profiles="plonetheme.sunburst:default" />',

                    '    <inflator:bundle',
                    '        title="bar"',
                    '        profiles="plonetheme.sunburst:default" />',

                    '</configure>'
                    )))

        utilites = self.get_utilites()
        self.assertEquals(len(utilites), 2)
        _name, foo = utilites[0]
        _name, bar = utilites[1]

        self.assertEquals(foo.title, u'foo')
        self.assertEquals(bar.title, u'bar')

    def test_reregistering_bundles(self):
        with self.assertRaises(ConfigurationConflictError):
            self.layer.load_zcml_string(
                '\n'.join((
                        '<configure xmlns:inflator="http://namespaces.zope.org/inflator"'
                        '           i18n_domain="my.package">',

                        '    <inflator:bundle',
                        '        title="foo"',
                        '        profiles="plonetheme.sunburst:default" />',

                        '    <inflator:bundle',
                        '        title="foo"',
                        '        profiles="plonetheme.sunburst:default" />',

                        '</configure>'
                        )))



from collective.transmogrifier.transmogrifier import Transmogrifier
from ftw.inflator.testing import INFLATOR_INTEGRATION_TESTING
from unittest2 import TestCase


class TestSingleItemContentCreation(TestCase):
    layer = INFLATOR_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']

    def test_create_single_item(self):
        item = {'_path': 'foo',
                '_type': 'Folder',
                'title': 'Foo'}

        mogrifier = Transmogrifier(self.portal)
        self.assertNotIn('foo', self.portal)
        mogrifier(u'ftw.inflator.creation.single_item_content_creation',
                  jsonsource=dict(item=item))
        self.assertIn('foo', self.portal)
        self.assertEquals('Foo', self.portal.foo.Title())



from collective.transmogrifier.interfaces import ISectionBlueprint
from ftw.inflator.creation.sections.base import ObjectUpdater
from persistent.list import PersistentList
from persistent.mapping import PersistentMapping
from zope.annotation.interfaces import IAnnotations
from zope.interface import classProvides


def make_persistent(data):
    if isinstance(data, dict):
        new = PersistentMapping()
        for key, value in data.items():
            new[make_persistent(key)] = make_persistent(value)
        return new

    elif isinstance(data, list):
        new = PersistentList()
        for value in data:
            new.append(make_persistent(value))
        return new

    else:
        return data


class AnnotationsUpdater(ObjectUpdater):
    classProvides(ISectionBlueprint)

    key_option_name = 'annotations-key'
    default_key_name = 'annotations'

    def update(self, obj, data):
        annotations = IAnnotations(obj)

        for key, value in data.items():
            annotations[make_persistent(key)] = make_persistent(value)



from Products.ATContentTypes.lib import constraintypes
from Products.CMFPlone.utils import base_hasattr
from collective.transmogrifier.interfaces import ISectionBlueprint
from ftw.inflator.creation.sections.base import ObjectUpdater
from zope.interface import classProvides


class ConstraintypesUpdater(ObjectUpdater):
    classProvides(ISectionBlueprint)

    key_option_name = 'constraintypes-key'
    default_key_name = 'constrain_types'

    def update(self, obj, data):
        if base_hasattr(obj, 'getConstrainTypesMode'):
            obj.setConstrainTypesMode(constraintypes.ENABLED)

            immediately = tuple(data['immediately'])
            obj.setImmediatelyAddableTypes(immediately)

            locally = tuple(data['locally'])
            obj.setLocallyAllowedTypes(locally)



from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from ftw.inflator.creation.sections import resolvetree
from unittest2 import TestCase
from zope.interface.verify import verifyClass
from zope.interface.verify import verifyObject


class TestResolveTreeBlueprint(TestCase):

    def test_implements_interface(self):
        klass = resolvetree.ResolveTree

        self.assertTrue(ISection.implementedBy(klass),
                        'Class %s does not implement ISection.' % str(klass))
        verifyClass(ISection, klass)

        self.assertTrue(ISectionBlueprint.providedBy(klass),
                        'Class %s does not provide ISectionBlueprint.' %
                        str(klass))
        verifyObject(ISectionBlueprint, klass)

    def test_resolves_tree_structure(self):
        input = [{
            '_path': 'foo',
            '_children': [{
                '_id': 'bar',
                'title': 'Bar',
                '_children': [{
                    '_id': 'baz'}]
            }, {
                '_path': '/qux/there',
                'title': 'Something'
            }]
        }]

        expected = [
            {'_path': 'foo'},
            {'_path': 'foo/bar',
             'title': 'Bar'},
            {'_path': 'foo/bar/baz'},
            {'_path': 'foo/qux/there',
             'title': 'Something'}
        ]

        source = resolvetree.ResolveTree(None, '', None, input)
        output = list(source)

        self.maxDiff = None
        self.assertEqual(output, expected)




def recursive_encode(data):
    """Encodes unicodes (from json) to utf-8 strings recursively.
    """
    if isinstance(data, unicode):
        return data.encode('utf-8')

    elif isinstance(data, str):
        return data

    elif isinstance(data, dict):
        for key, value in data.items():
            del data[key]
            data[recursive_encode(key)] = recursive_encode(value)
        return data

    elif isinstance(data, list):
        new_data = []
        for item in data:
            new_data.append(recursive_encode(item))
        return new_data

    elif hasattr(data, '__iter__'):
        for item in data:
            recursive_encode(item)
        return data

    else:
        return data



from ftw.inflator.interfaces import IInflatorCustomization
from zope.component import getUtilitiesFor
from zope.interface import implements


def get_merged_customizations():
    customizations = sorted(getUtilitiesFor(IInflatorCustomization))

    custom_data = {}

    for _order, cust in customizations:
        data = vars(cust).copy()
        del data['order']
        data = dict([(key, value) for key, value
                     in data.items()
                     if value is not None])
        custom_data.update(data)

    return InflatorCustomization(**custom_data)


class InflatorCustomization(object):
    """Inflator customization utility.
    """

    implements(IInflatorCustomization)

    def __init__(self, product=None, image=None, order=10):
        self.product = product
        self.image = image
        self.order = order



from ftw.inflator.creation.sections import interfaces
from ftw.inflator.tests.interfaces import IFoo
from ftw.testing import MockTestCase
from zope.interface import alsoProvides


class TestInterfacesUpdater(MockTestCase):

    def setUp(self):
        super(TestInterfacesUpdater, self).setUp()
        transmogrifier = self.create_dummy(context=None)
        options = {'blueprint': ''}
        self.updater = interfaces.InterfacesUpdater(
            transmogrifier, '', options, [])

    def test_interface_also_provided(self):
        obj = self.create_dummy()
        self.assertFalse(IFoo.providedBy(obj))

        data = ['ftw.inflator.tests.interfaces.IFoo']
        self.updater.update(obj, data)
        self.assertTrue(IFoo.providedBy(obj))

    def test_interface_not_reprovided(self):
        obj = self.providing_stub([IFoo])
        self.replay()

        data = ['ftw.inflator.tests.interfaces.IFoo']
        self.updater.update(obj, data)

    def test_remove_interface(self):
        obj = self.create_dummy()
        alsoProvides(obj, IFoo)
        self.assertTrue(IFoo.providedBy(obj))

        data = ['remove:ftw.inflator.tests.interfaces.IFoo']
        self.updater.update(obj, data)
        self.assertFalse(IFoo.providedBy(obj))



from ftw.inflator.bundle import get_bundles
from ftw.inflator.testing import META_ZCML
from ftw.testing import MockTestCase


class TestBundlesZCML(MockTestCase):

    layer = META_ZCML

    def test_getting_bundles_with_no_bundles_registered(self):
        self.assertEquals(get_bundles(), [])

    def test_getting_bundles(self):
        self.layer.load_zcml_string('\n'.join((
                    '<configure xmlns:inflator="http://namespaces.zope.org/inflator"'
                    '           i18n_domain="my.package">',

                    '    <inflator:bundle',
                    '        title="foo"',
                    '        profiles="plonetheme.sunburst:default" />',

                    '    <inflator:bundle',
                    '        title="bar"',
                    '        profiles="plonetheme.sunburst:default" />',

                    '</configure>'
                    )))

        bundles = get_bundles()
        self.assertEquals(type(bundles), list)
        self.assertEquals(len(bundles), 2)

        bar, foo = bundles

        self.assertEquals(foo.title, u'foo')
        self.assertEquals(bar.title, u'bar')

    def test_bundle_sorting(self):
        # 1. non-standard first, then standard
        # 2. sort by title

        self.layer.load_zcml_string('\n'.join((
                    '<configure xmlns:inflator="http://namespaces.zope.org/inflator"'
                    '           i18n_domain="my.package">',

                    '    <inflator:bundle',
                    '        title="foo-standard"',
                    '        profiles="plonetheme.sunburst:default"'
                    '        standard="True" />',

                    '    <inflator:bundle',
                    '        title="bar"',
                    '        profiles="plonetheme.sunburst:default" />',

                    '    <inflator:bundle',
                    '        title="foo"',
                    '        profiles="plonetheme.sunburst:default" />',

                    '    <inflator:bundle',
                    '        title="bar-standard"',
                    '        profiles="plonetheme.sunburst:default"'
                    '        standard="True" />',

                    '</configure>'
                    )))

        bundles = get_bundles()
        self.assertEquals(type(bundles), list)
        self.assertEquals(len(bundles), 4)

        ordered_titles = [bundle.title for bundle in bundles]
        self.assertEqual(ordered_titles,
                         [u'bar',
                          u'foo',
                          u'bar-standard',
                          u'foo-standard'])



from ftw.inflator.creation.sections import schemaupdater
from ftw.testing import MockTestCase


class ArchetypesFTI(object):
    pass


class DexterityFTI(object):
    pass


class TestNoDexteritySupportUpdateSection(MockTestCase):

    def setUp(self):
        super(TestNoDexteritySupportUpdateSection, self).setUp()
        self.transmogrifier = self.create_dummy(context=None)

    def test_yields_non_dexterity_types(self):
        portal_types = self.stub()
        fti = ArchetypesFTI()
        self.expect(portal_types.get('SomeType')).result(fti)
        self.mock_tool(portal_types, 'portal_types')
        self.replay()

        item = {'_type': 'SomeType'}
        updater = schemaupdater.NoDexteritySupportUpdateSection(
            self.transmogrifier, '', {}, [item])

        self.assertEqual(list(updater), [item])

    def test_raises_on_dexterity_types(self):
        portal_types = self.stub()
        fti = DexterityFTI()
        self.expect(portal_types.get('OtherType')).result(fti)
        self.mock_tool(portal_types, 'portal_types')
        self.replay()

        item = {'_type': 'OtherType'}
        updater = schemaupdater.NoDexteritySupportUpdateSection(
            self.transmogrifier, '', {}, [item])

        with self.assertRaises(ValueError) as cm:
            list(updater)

        self.assertEqual(
            str(cm.exception),
            'Trying to update a dexterity object of type OtherType but'
            ' transmogrify.dexterity is not installed'
            " ({'_type': 'OtherType'})")



from Products.CMFCore.utils import getToolByName
from plone.i18n.normalizer.interfaces import IURLNormalizer
from zope.component import queryUtility
from zope.i18n.locales import locales


def setup_language(portal):
    """When installing plone, the language is set when creating
    example content.
    If the content creation profile ``Products.CMFPlone:plone-content``
    is not installed, the lanuage is not set up properly.

    This setup handler allows to only setup the language but not
    create example content by putting a ``inflator-setup-language.txt``
    in a generic setup profile which is used on initialization.

    The code of this function is copied from
    ``Products.CMFPlone.setuphandlers.setupPortalContent``.
    """
    language = portal.Language()
    parts = (language.split('-') + [None, None])[:3]
    locale = locales.getLocale(*parts)
    target_language = base_language = locale.id.language

    # If we get a territory, we enable the combined language codes
    use_combined = False
    if locale.id.territory:
        use_combined = True
        target_language += '_' + locale.id.territory

    # As we have a sensible language code set now, we disable the
    # start neutral functionality
    tool = getToolByName(portal, "portal_languages")

    tool.manage_setLanguageSettings(language,
        [language],
        setUseCombinedLanguageCodes=use_combined,
        startNeutral=False)

    # Set the first day of the week, defaulting to Sunday, as the
    # locale data doesn't provide a value for English. European
    # languages / countries have an entry of Monday, though.
    calendar = getToolByName(portal, "portal_calendar", None)
    if calendar is not None:
        first = 6
        gregorian = locale.dates.calendars.get(u'gregorian', None)
        if gregorian is not None:
            first = gregorian.week.get('firstDay', None)
            # on the locale object we have: mon : 1 ... sun : 7
            # on the calendar tool we have: mon : 0 ... sun : 6
            if first is not None:
                first = first - 1

        calendar.firstweekday = first

    # Enable visible_ids for non-latin scripts

    # See if we have an url normalizer
    normalizer = queryUtility(IURLNormalizer, name=target_language)
    if normalizer is None:
        normalizer = queryUtility(IURLNormalizer, name=base_language)



from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from collective.transmogrifier.utils import defaultMatcher
from zope.interface import classProvides
from zope.interface import implements


class BlockLocalRoleInheritance(object):
    """Block the inheritance of ac_local_roles.
    """

    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.context = transmogrifier.context
        self.pathkey = defaultMatcher(options, 'path-key', name, 'path')
        self.block_local_roles_key = defaultMatcher(options,
                                                    'block-local-roles-key',
                                                    name,
                                                    'block-local-roles')

    def __iter__(self):
        for item in self.previous:
            self.set_block_local_roles(item)
            yield item

    def get_item_value(self, item, key):
        return item.get(key(*item.keys())[0], None)

    def set_block_local_roles(self, item):
        path = self.get_item_value(item, self.pathkey)
        if not path:
            return

        obj = self.context.unrestrictedTraverse(path.lstrip('/'), None)
        if not obj:
            return

        block_local_roles = self.get_item_value(item,
                                                self.block_local_roles_key)
        if not block_local_roles:
            return

        obj.__ac_local_roles_block__ = True
        obj.reindexObjectSecurity()



from collective.transmogrifier.interfaces import ISectionBlueprint
from ftw.inflator.creation.sections.base import ObjectUpdater
from zope.dottedname.resolve import resolve
from zope.interface import alsoProvides
from zope.interface import classProvides
from zope.interface import noLongerProvides


class InterfacesUpdater(ObjectUpdater):
    classProvides(ISectionBlueprint)

    key_option_name = 'interfaces-key'
    default_key_name = 'interfaces'

    def update(self, obj, interfaces):
        for iface in interfaces:
            iface = iface.encode('utf-8')
            remove = False

            if iface.startswith('remove:'):
                remove = True
                iface = iface[len('remove:'):]

            resolved_iface = resolve(iface)

            if not remove and not resolved_iface.providedBy(obj):
                alsoProvides(obj, resolved_iface)

            elif remove and resolved_iface.providedBy(obj):
                noLongerProvides(obj, resolved_iface)



from ftw.inflator.testing import ZOPE_FUNCTIONAL_TESTING
from plone.app.testing import SITE_OWNER_NAME, SITE_OWNER_PASSWORD
from plone.testing import zca
from plone.testing.z2 import Browser
from plone.testing.z2 import zopeApp
from unittest2 import TestCase
from zope.configuration import xmlconfig


class TestZMIButton(TestCase):

    layer = ZOPE_FUNCTIONAL_TESTING

    def setUp(self):
        zca.pushGlobalRegistry()
        self.configurationContext = zca.stackConfigurationContext()

        import ftw.inflator
        xmlconfig.file('meta.zcml', ftw.inflator, context=self.configurationContext)

        with zopeApp() as app:
            self.app = app

            self.browser = Browser(app)
            self.browser.handleErrors = False
            self.browser.addHeader('Authorization', 'Basic %s:%s' % (
                    SITE_OWNER_NAME, SITE_OWNER_PASSWORD))

    def tearDown(self):
        zca.popGlobalRegistry()
        self.configurationContext = None

    def load_zcml_string(self, zcml):
        xmlconfig.string(zcml, context=self.configurationContext)

    def test_add_plone_site_button_still_tehere(self):
        self.browser.open('http://localhost/manage_main')
        self.assertTrue(self.browser.getControl('Add Plone Site'))

    def test_default_inflator_install_plone_buttton(self):
        self.browser.open('http://localhost/manage_main')
        self.assertTrue(self.browser.getControl('Install Plone'))

    def test_customized_inflator_install_buttton(self):
        self.load_zcml_string('\n'.join((
                    '<configure xmlns:inflator="http://namespaces.zope.org/inflator"'
                    '           i18n_domain="my.package">',
                    '    <inflator:customize',
                    '        product="my product" />',
                    '</configure>'
                    )))

        self.browser.open('http://localhost/manage_main')
        self.assertTrue(self.browser.getControl('Install my product'))



from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from ftw.inflator.creation.sections import jsonsource
from unittest2 import TestCase
from zope.interface.verify import verifyClass
from zope.interface.verify import verifyObject
import ftw.inflator.tests
import os.path


class TestJSONSource(TestCase):

    def test_implements_interface(self):
        klass = jsonsource.JSONSource

        self.assertTrue(ISection.implementedBy(klass),
                        'Class %s does not implement ISection.' % str(klass))
        verifyClass(ISection, klass)

        self.assertTrue(ISectionBlueprint.providedBy(klass),
                        'Class %s does not provide ISectionBlueprint.' % (
                str(klass)))
        verifyObject(ISectionBlueprint, klass)

    def test_directory_checks(self):
        existing_dir = os.path.dirname(jsonsource.__file__)
        missing_dir = os.path.join(existing_dir, 'foo-missing-dir')

        self.assertFalse(
            os.path.isdir(missing_dir),
            'Test not possible since unexpected directory present at %s' % (
                missing_dir))

        jsonsource.JSONSource(None, '', {'directory': existing_dir}, [])

        with self.assertRaises(IOError) as cm:
            jsonsource.JSONSource(None, '', {'directory': missing_dir}, [])

        self.assertEqual(str(cm.exception),
                         'Directory does not exists: %s' % missing_dir)

    def test_reiters_previous(self):
        directory = os.path.dirname(ftw.inflator.tests.__file__)

        source = jsonsource.JSONSource(
            None, '', {'directory': directory}, ['foo', 'bar'])

        self.assertEqual(list(source), ['foo', 'bar'])

    def test_json_file_parsing(self):
        directory = os.path.join(
            os.path.dirname(ftw.inflator.tests.__file__),
            'jsonsource')

        source = jsonsource.JSONSource(
            None, '', {'directory': directory}, ['previous'])

        self.assertEqual(list(source), ['previous', 'foo', 1, 'b\xc3\xa4r'])



from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.utils import defaultMatcher
from zope.interface import implements


class ObjectUpdater(object):

    implements(ISection)

    key_option_name = None
    default_key_name = None

    def __init__(self, transmogrifier, name, options, previous):
        self.transmogrifier = transmogrifier
        self.name = name
        self.options = options
        self.previous = previous
        self.context = transmogrifier.context
        self.datakey = defaultMatcher(
            options, self.key_option_name, name, self.default_key_name)
        self.pathkey = defaultMatcher(options, 'path-key', name, 'path')

    def __iter__(self):
        for item in self.previous:
            data = item.get(self.datakey(*item.keys())[0], None)
            obj = self._get_obj(item)
            if obj and data:
                self.update(obj, data)

            yield item

    def _get_obj(self, item):
        path = item.get(self.pathkey(*item.keys())[0], None)
        # Skip the Plone site object itself
        if not path:
            return None

        obj = self.context.unrestrictedTraverse(
            path.encode('utf-8').lstrip('/'), None)

        return obj

    def update(self, obj, data):
        raise NotImplementedError()



from collective.transmogrifier.interfaces import ISectionBlueprint
from ftw.inflator.creation.sections.base import ObjectUpdater
from zope.interface import classProvides


class PropertiesUpdater(ObjectUpdater):
    classProvides(ISectionBlueprint)

    key_option_name = 'properties-key'
    default_key_name = 'properties'

    def update(self, obj, properties):
        for key, data in properties.items():
            type_, value = data

            if obj.getProperty(key):
                obj._updateProperty(key, value)
            else:
                obj._setProperty(key, value, type_)



from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from collective.transmogrifier.utils import resolvePackageReferenceOrFile
from ftw.inflator.creation.progresslogger import ProgressLogger
from ftw.inflator.creation.sections.utils import recursive_encode
from zope.interface import classProvides
from zope.interface import implements
import json
import os.path


class JSONSource(object):
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous

        self.path = resolvePackageReferenceOrFile(options['directory'])
        if self.path is None or not os.path.isdir(self.path):
            raise IOError('Directory does not exists: %s' % self.path)

    def __iter__(self):
        for item in self.previous:
            yield item

        msg = 'Importing content_creation files with inflator.'
        for name in ProgressLogger(msg, sorted(os.listdir(self.path))):
            if name.startswith('.') or not name.endswith('.json'):
                continue

            path = os.path.join(self.path, name)
            if not os.path.isfile(path):
                continue

            locals()['__traceback_info__'] = path

            with open(path) as file_:
                data = json.loads(file_.read())

            for item in data:
                # yield item
                yield recursive_encode(item)



from ftw.inflator.creation.sections import properties
from ftw.testing import MockTestCase


class TestPropertiesUpdater(MockTestCase):

    def setUp(self):
        super(TestPropertiesUpdater, self).setUp()
        transmogrifier = self.create_dummy(context=None)
        options = {'blueprint': ''}
        self.updater = properties.PropertiesUpdater(
            transmogrifier, '', options, [])

    def test_create_properties(self):
        obj = self.mocker.mock()
        self.expect(obj.getProperty('foo')).result(None)
        self.expect(obj._setProperty('foo', 'bar', 'string'))
        self.replay()

        data = {'foo': ['string', 'bar']}
        self.updater.update(obj, data)

    def test_update_properties(self):
        obj = self.mocker.mock()
        self.expect(obj.getProperty('foo')).result('something')
        self.expect(obj._updateProperty('foo', 'bar'))
        self.replay()

        data = {'foo': ['string', 'bar']}
        self.updater.update(obj, data)



from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from ftw.inflator.creation.sections import singleitemsource
from unittest2 import TestCase
from zope.interface.verify import verifyClass
from zope.interface.verify import verifyObject


class TestSingleItemSource(TestCase):

    def test_implements_interface(self):
        klass = singleitemsource.SingleItemSource

        self.assertTrue(ISection.implementedBy(klass),
                        'Class %s does not implement ISection.' % str(klass))
        verifyClass(ISection, klass)

        self.assertTrue(ISectionBlueprint.providedBy(klass),
                        'Class %s does not provide ISectionBlueprint.' % (
                str(klass)))
        verifyObject(ISectionBlueprint, klass)

    def test_yields_item_from_options(self):
        source = singleitemsource.SingleItemSource(
            None, '', {'item': {'foo': 'bar'}}, ())

        self.assertEqual([{'foo': 'bar'}], list(source))



from Products.CMFCore.utils import getToolByName
from ftw.inflator.testing import ZOPE_FUNCTIONAL_TESTING
from plone.app.testing import SITE_OWNER_NAME, SITE_OWNER_PASSWORD
from plone.testing.z2 import Browser
from plone.testing.z2 import zopeApp
from unittest2 import TestCase


class TestInflateView(TestCase):

    layer = ZOPE_FUNCTIONAL_TESTING

    def setUp(self):
        with zopeApp() as app:
            self.app = app

            self.browser = Browser(app)
            self.browser.handleErrors = False
            self.browser.addHeader('Authorization', 'Basic %s:%s' % (
                    SITE_OWNER_NAME, SITE_OWNER_PASSWORD))

    def test_add_plone_site_button_still_tehere(self):
        self.browser.open('http://localhost/@@inflate')
        self.assertIn('++resource++inflator-plone-logo-0.png',
                      self.browser.contents)

    def test_default_form_values(self):
        self.browser.open('http://localhost/@@inflate')

        # default values
        self.assertEquals(self.browser.getControl(name='site_id').value,
                          'platform')

        self.assertEquals(self.browser.getControl(name='title').value,
                          'Plone')

        self.assertEquals(
            self.browser.getControl(name='default_language').value,
            ['en'])

        self.assertEquals(
            self.browser.getControl(name='bundle').value,
            ['ftw.inflator example bundle one'])

    def test_install_test_profile(self):
        self.browser.open('http://localhost/@@inflate')

        self.browser.getControl(name='default_language').value = ['de']
        self.browser.getControl('Install').click()
        self.assertEqual(self.browser.url, 'http://localhost/platform')

        site = self.app.get('platform')
        self.assertTrue(site)

        language_tool = getToolByName(site, 'portal_languages')
        self.assertEqual(language_tool.getDefaultLanguage(), 'de')



from ftw.inflator.customization import get_merged_customizations
from ftw.inflator.testing import ZOPE_LAYER
from plone.testing import z2
from plone.testing import zca
from unittest2 import TestCase
from zope.configuration import xmlconfig


class TestCustomizationZCML(TestCase):

    layer = ZOPE_LAYER

    def setUp(self):
        zca.pushGlobalRegistry()
        self.configurationContext = zca.stackConfigurationContext()

        import ftw.inflator
        xmlconfig.file('meta.zcml', ftw.inflator, context=self.configurationContext)

    def tearDown(self):
        zca.popGlobalRegistry()
        self.configurationContext = None

    def load_zcml_string(self, zcml):
        xmlconfig.string(zcml, context=self.configurationContext)

    def test_customize_product_name(self):
        self.load_zcml_string('\n'.join((
                    '<configure xmlns:inflator="http://namespaces.zope.org/inflator"'
                    '           i18n_domain="my.package">',
                    '    <inflator:customize',
                    '        product="my product" />',
                    '</configure>'
                    )))

        self.assertEquals(get_merged_customizations().product, u'my product')

    def test_customize_image(self):
        self.load_zcml_string('\n'.join((
                    '<configure xmlns:inflator="http://namespaces.zope.org/inflator"'
                    '           i18n_domain="my.package"'
                    '           package="ftw.inflator.tests">',
                    '    <inflator:customize',
                    '        image="../resources/plone-logo.png" />',
                    '</configure>'
                    )))

        image = get_merged_customizations().image
        self.assertEquals(image, '++resource++inflator-plone-logo-10.png')

        with z2.zopeApp() as app:
            self.assertTrue(app.unrestrictedTraverse(image))

    def test_merging_cusetomizations(self):
        self.load_zcml_string('\n'.join((
                    '<configure xmlns:inflator="http://namespaces.zope.org/inflator"'
                    '           i18n_domain="my.package"'
                    '           package="ftw.inflator.tests">',

                    '    <inflator:customize',
                    '        product="foo"'
                    '        image="../resources/plone-logo.png"'
                    '        order="4"/>',

                    '    <inflator:customize',
                    '        product="bar"'
                    '        order="5"/>',

                    '</configure>'
                    )))

        customizations = get_merged_customizations()
        self.assertEquals(customizations.image, '++resource++inflator-plone-logo-4.png')
        self.assertEquals(customizations.product, 'bar')

    def test_defaults(self):
        customizations = get_merged_customizations()
        self.assertEquals(customizations.image, '++resource++inflator-plone-logo-0.png')
        self.assertEquals(customizations.product, 'Plone')



from ftw.inflator.customization import InflatorCustomization
from ftw.inflator.interfaces import IInflatorCustomization
from unittest2 import TestCase
from zope.interface.verify import verifyClass


class TestCustomizationUtility(TestCase):

    def test_customization_implements_interface(self):
        self.assertTrue(IInflatorCustomization.implementedBy(InflatorCustomization))
        verifyClass(IInflatorCustomization, InflatorCustomization)

    def test_stores_arguments(self):
        obj = InflatorCustomization(product=u'foo',
                                    image='++resource++bar.jpg',
                                    order=7)

        self.assertEqual(obj.product, u'foo')
        self.assertEqual(obj.image, '++resource++bar.jpg')
        self.assertEqual(obj.order, 7)

    def test_defaults(self):
        obj = InflatorCustomization()

        self.assertEqual(obj.product, None)
        self.assertEqual(obj.image, None)
        self.assertEqual(obj.order, 10)



import datetime
from Products.CMFCore.utils import getToolByName
from ftw.inflator.testing import INFLATOR_FIXTURE
from ftw.inflator.tests.interfaces import IFoo, IExampleDxType
from persistent.list import PersistentList
from persistent.mapping import PersistentMapping
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import applyProfile
from unittest2 import TestCase
from zope.annotation.interfaces import IAnnotations
from zope.configuration import xmlconfig


class DXCreationLayer(PloneSandboxLayer):

    defaultBases = (INFLATOR_FIXTURE, )

    def setUpZope(self, app, configurationContext):
        import plone.app.dexterity
        xmlconfig.file('configure.zcml', plone.app.dexterity,
                       context=configurationContext)

    def setUpPloneSite(self, portal):
        wftool = getToolByName(portal, 'portal_workflow')
        wftool.setChainForPortalTypes(['ExampleDxType'],
                                      'simple_publication_workflow')

        applyProfile(portal, 'ftw.inflator:setup-language')
        applyProfile(portal, 'ftw.inflator.tests:dx_creation')


DX_FIXTURE = DXCreationLayer()
DX_INTEGRATION_TESTING = IntegrationTesting(
    bases=(DX_FIXTURE, ), name="ftw.inflator:DX Integration")


class TestDXContentCreation(TestCase):

    layer = DX_INTEGRATION_TESTING

    def setUp(self):
        super(TestDXContentCreation, self).setUp()
        self.portal = self.layer['portal']

    def test_object_is_there(self):
        self.assertTrue(self.portal.get('my-dx'))

    def test_title(self):
        obj = self.portal.get('my-dx')
        self.assertEquals(obj.Title(), 'My Dexterity Object')

    def test_catalog(self):
        obj_path = '/'.join(self.portal.get('my-dx').getPhysicalPath())
        query = {'path': {'query': obj_path,
                          'depth': 0}}

        catalog = getToolByName(self.portal, 'portal_catalog')
        brains = catalog.unrestrictedSearchResults(query)
        self.assertEqual(len(brains), 1)

        obj_brain = brains[0]
        self.assertEqual(obj_brain.portal_type, 'ExampleDxType')
        self.assertEqual(obj_brain.Title, 'My Dexterity Object')

    def test_richtext_inline_inserted(self):
        obj = self.portal.get('richtext-inlined')
        self.assertEquals(
            u'<p>This HTML is inlined.</p>',
            IExampleDxType(obj).richtext.output)

        obj = self.portal.get('richtext-file-embedded')
        self.assertEquals(
            u'<p>This HTML ist stored in a separate richtext.html file.</p>',
            IExampleDxType(obj).richtext.output.strip())

    def test_properties(self):
        obj = self.portal.get('my-dx')
        self.assertEqual(obj.getProperty('f\xc3\xbc\xc3\xbc'), 'b\xc3\xa4r')

    def test_interfaces(self):
        obj = self.portal.get('my-dx')

        self.assertTrue(IFoo.providedBy(obj),
                        'IFoo interface not provided by Object my-dx')

    def test_annotations(self):
        obj = self.portal.get('my-dx')
        annotations = IAnnotations(obj)
        self.assertEqual(annotations.get('foo'), {
                'bar': [1, 2, 'three'],
                'baz': True})

        self.assertFalse(isinstance({}, PersistentMapping))
        self.assertTrue(isinstance(annotations.get('foo'),
                                   PersistentMapping))

        self.assertFalse(isinstance({}, PersistentList))
        self.assertTrue(isinstance(annotations.get('foo').get('bar'),
                                   PersistentList))

    def test_local_roles_are_created(self):
        obj = self.portal.get('my-dx')
        roles = dict(obj.get_local_roles())
        self.assertEqual(("Owner", "Contributor", "Editor",),
                         roles.get("admin"))
        self.assertEqual(("Editor",), roles.get("hans"))

    def test_paths_are_resolved(self):
        obj = self.portal.get('my-dx')
        target = self.portal.get('target')
        self.assertEqual(obj.relation, target)

    def test_date(self):
        obj = self.portal.get('my-dx')
        self.assertEqual(
            datetime.date(2015, 1, 1),
            IExampleDxType(obj).date
        )

    def test_datetime(self):
        obj = self.portal.get('my-dx')
        self.assertEqual(
            datetime.datetime(2015, 1, 1, 15, 0, 59),
            IExampleDxType(obj).datetime
        )

    def test_filename_can_be_changed(self):
        obj = self.portal.get('filename-changed')
        image = IExampleDxType(obj).image
        self.assertEquals('filename-has-changed.jpg', image.filename)



from Products.CMFCore.utils import getToolByName
from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from zope.interface import classProvides
from zope.interface import implements
import pkg_resources


try:
    pkg_resources.get_distribution('transmogrify.dexterity')

except pkg_resources.DistributionNotFound:
    DEXTERITY_SUPPORT = False

else:
    from transmogrify.dexterity.schemaupdater import DexterityUpdateSection
    DEXTERITY_SUPPORT = True


class NoDexteritySupportUpdateSection(object):
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.context = transmogrifier.context
        self.previous = previous

    def __iter__(self):
        ttool = getToolByName(self.context, 'portal_types')

        for item in self.previous:
            fti = ttool.get(item.get('_type'))
            if fti and fti.__class__.__name__ == 'DexterityFTI':
                raise ValueError('Trying to update a dexterity object of'
                                 ' type %s but transmogrify.dexterity is'
                                 ' not installed (%s)' % (
                        item.get('_type'),
                        str(item)))

            yield item


if DEXTERITY_SUPPORT:
    DxUpdateSection = DexterityUpdateSection
else:
    DxUpdateSection = NoDexteritySupportUpdateSection



from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from ftw.inflator.creation.sections.utils import recursive_encode
from zope.interface import classProvides
from zope.interface import implements


class SingleItemSource(object):
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.options = options

    def __iter__(self):
        for item in self.previous:
            yield item

        yield recursive_encode(self.options['item'])



from Products.CMFCore.utils import getToolByName
from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from copy import deepcopy
from ftw.inflator.creation.sections.base import ObjectUpdater
from ftw.inflator.exceptions import MultilingalInflateException
from plone.uuid.interfaces import IUUIDGenerator
from zope.component import getUtility
from zope.interface import classProvides
from zope.interface import implements
import pkg_resources


try:
    pkg_resources.get_distribution('plone.app.multilingual')

except pkg_resources.DistributionNotFound:
    HAS_MULTILINGUAL = False

else:
    HAS_MULTILINGUAL = True
    from plone.app.multilingual.browser.setup import SetupMultilingualSite
    try:
        # plone.app.multilingual >= 2.x
        from plone.app.multilingual.interfaces import ILanguage
        from plone.app.multilingual.interfaces import IMutableTG
        from plone.app.multilingual.interfaces import ITranslationManager
    except:
        # plone.app.multilingual 1.x
        from plone.multilingual.interfaces import ILanguage
        from plone.multilingual.interfaces import IMutableTG
        from plone.multilingual.interfaces import ITranslationManager


class SetupLanguages(object):
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.previous = previous
        self.context = transmogrifier.context

    def __iter__(self):
        for item in self.previous:
            if '_multilingual' in item:
                for subitem in self.setup_languages(item):
                    yield subitem
            else:
                yield item

    def setup_languages(self, item):
        if not HAS_MULTILINGUAL:
            raise MultilingalInflateException(
                "Defined multilingal content but plone.app.multilingual is "
                "not installed")

        self._validate_languages(item['_multilingual'])

        multilingual_setup = SetupMultilingualSite()
        multilingual_setup.setupSite(self.context)

        contents = deepcopy(item['_contents'])
        self._recursive_create_translation_group(contents)

        for lang_code in item['_multilingual']:
            lang_contents = deepcopy(contents)
            self._recursive_set_language(lang_contents, lang_code)

            subitem = {'_path': lang_code,
                       '_children': lang_contents}
            if '_properties' in item:
                subitem['_properties'] = deepcopy(item['_properties'])
            yield subitem

    def _validate_languages(self, languages):
        portal_languages = getToolByName(self.context, 'portal_languages')

        msg = ("Language '{}' is not configured. Configured languages "
               "are: '{}'. Check if portal_languages.xml is available and "
               "if it contains all required languages.")
        for lang_code in languages:
            if lang_code not in portal_languages.supported_langs:
                raise MultilingalInflateException(msg.format(lang_code,
                                ','.join(portal_languages.supported_langs)))

    def _recursive_create_translation_group(self, items):
        uuid_generator = getUtility(IUUIDGenerator)

        for item in items:
            item['_multilingual_settings'] = {
                'translation_group_uuid': uuid_generator()}

            if '_children' in item:
                self._recursive_create_translation_group(item['_children'])

    def _recursive_set_language(self, items, lang_code):
        for item in items:
            item['_multilingual_settings']['language'] = lang_code
            if '_children' in item:
                self._recursive_set_language(item['_children'], lang_code)


class LinkMultilingualContent(ObjectUpdater):
    classProvides(ISectionBlueprint)

    key_option_name = 'multilingual-settings-key'
    default_key_name = 'multilingual_settings'

    def update(self, obj, data):
        ILanguage(obj).set_language(data['language'])
        IMutableTG(obj).set(data['translation_group_uuid'])

        manager = ITranslationManager(obj)
        manager.register_translation(data['language'], obj)



from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from zope.i18n import translate
from zope.interface import classProvides
from zope.interface import implements
import re


TRANSLATABLE_KEY_EXPR = re.compile(
    r'^([^:]*):translate\(([^)]*)\)$')


class Translate(object):
    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.target_language = transmogrifier.context.Language()
        self.previous = previous

    def __iter__(self):
        for item in self.previous:
            item = self.translate_recursive(item, self.target_language)
            yield item

    def translate_recursive(self, data, language):
        if isinstance(data, list):
            new_data = []
            for item in data:
                new_data.append(self.translate_recursive(item, language))

            return data

        if not isinstance(data, dict):
            return data

        if '_multilingual_settings' in data:
            language = data['_multilingual_settings']['language']

        for key, value in data.items():
            if isinstance(value, str):
                value = value.decode('utf-8')
            value = self.translate_recursive(value, language)

            match = TRANSLATABLE_KEY_EXPR.match(key)
            if not match:
                data[key] = value
                continue

            new_key, domain = match.groups()
            data[new_key] = translate(value, domain=domain,
                                      target_language=language)
            del data[key]

        return data



from ftw.inflator.creation.sections.utils import recursive_encode
from unittest2 import TestCase


class TestRecursiveEncode(TestCase):

    def equals(self, input, output):
        self.assertEqual(recursive_encode(input), output)

    def test_unicode(self):
        self.equals(u'foo', 'foo')
        self.equals(u'b\xe4r', 'b\xc3\xa4r')
        self.equals('b\xc3\xa4r', 'b\xc3\xa4r')

    def test_dicts(self):
        self.equals({u'b\xe4r': u'b\xe4r'},
                    {'b\xc3\xa4r': 'b\xc3\xa4r'})

    def test_lists(self):
        self.equals([u'b\xe4r'], ['b\xc3\xa4r'])

    def test_other_types(self):
        self.equals([1, 2.2, True, False, None],
                    [1, 2.2, True, False, None])

        self.equals(1, 1)
        self.equals(None, None)

    def test_nesting(self):
        input = {
            u'b\xe4r': {
                u'b\xe4r': [
                    u'b\xe4r',
                    1,
                    True]},
            'blubb': [{
                    u'b\xe4r': 3}]}

        output = {
            'b\xc3\xa4r': {
                'b\xc3\xa4r': [
                    'b\xc3\xa4r',
                    1,
                    True]},
            'blubb': [{
                    'b\xc3\xa4r': 3}]}

        self.equals(input, output)





def map_recursive(condition, function, sequence, types=(str, unicode)):
    """Walks recursively through a sequence and calls for each keys
    and values matching `types` (defautls to `(str, unicode)`) the
    passed `condition`.
    If the return value of the `condition` is positive it calls `function`
    and replaces the return value of the call in the structure.

    Example:

        >>> data = [{'foo': ['upper::bar', 'upper::baz']}]
        >>> result = map_recursive(
        ...     lambda item: item.startswith('upper::'),
        ...     lambda item: item.lstrip('upper::').upper(),
        ...     data)
        >>> data == result
        True
        >>> result
        [{'foo': ['BAR', 'BAZ']}]
    """

    if isinstance(sequence, types) and condition(sequence):
        return function(sequence)

    elif isinstance(sequence, list):
        for i, item in enumerate(sequence[:]):
            sequence[i:i + 1] = [map_recursive(
                    condition, function, item, types=types)]

    elif isinstance(sequence, tuple):
        return tuple(map_recursive(condition, function,
                                         list(sequence), types=types))

    elif isinstance(sequence, dict):
        for key, value in sequence.items():
            value = map_recursive(
                condition, function, value, types=types)

            new_key = map_recursive(
                condition, function, key, types=types)

            if new_key != key:
                del sequence[key]

            sequence[new_key] = value

    return sequence



from OFS.Image import File
from Products.CMFCore.utils import getToolByName
from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from zope.interface import classProvides
from zope.interface import implements
import os


class FileInserter(object):

    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.transmogrifier = transmogrifier
        self.name = name
        self.options = options
        self.previous = previous
        self.context = transmogrifier.context
        self.ttool = getToolByName(self.context, 'portal_types')

    def __iter__(self):
        for item in self.previous:
            for key in item.keys():
                if key.endswith(':file'):
                    self.load_file(item, key)

            yield item

    def load_file(self, item, key):
        fieldname = key[:-len(':file')]
        directory = self.transmogrifier.get('jsonsource').get('directory')
        path = os.path.join(directory, item.get(key))

        filename_key = '{}:filename'.format(fieldname)
        if filename_key in item:
            filename = item[filename_key]
        else:
            filename = os.path.basename(path)

        file_ = open(path, 'rb')
        try:
            fti = self.ttool.get(item.get('_type'))
            if fti and fti.__class__.__name__ == 'DexterityFTI':
                self.add_dx_file(item, fieldname, filename, file_)
            else:
                self.add_at_file(item, fieldname, filename, file_)
        finally:
            file_.close()

    def add_dx_file(self, item, fieldname, filename, file_):
        item[fieldname] = file_.read()
        # surprisingly this does not have a filename prefix/postfix so it
        # might only work with one file.
        item['_filename'] = filename

    def add_at_file(self, item, fieldname, filename, file_):
        item[fieldname] = File(filename, filename, file_)
        setattr(item[fieldname], 'filename', filename)



from ftw.inflator.creation.sections import annotations
from ftw.testing import MockTestCase
from persistent.list import PersistentList
from persistent.mapping import PersistentMapping
from zope.annotation.attribute import AttributeAnnotations
from zope.annotation.interfaces import IAnnotations
from zope.annotation.interfaces import IAttributeAnnotatable
from zope.component import provideAdapter
from zope.interface import alsoProvides


class TestAnnotationsUpdater(MockTestCase):

    def setUp(self):
        super(TestAnnotationsUpdater, self).setUp()
        transmogrifier = self.create_dummy(context=None)
        options = {'blueprint': ''}
        self.updater = annotations.AnnotationsUpdater(
            transmogrifier, '', options, [])

        provideAdapter(AttributeAnnotations,
                       adapts=(IAttributeAnnotatable,))

    def test_simple_annotation(self):
        obj = self.create_dummy()
        alsoProvides(obj, IAttributeAnnotatable)

        data = {'foo': 'bar'}
        self.updater.update(obj, data)

        self.assertEquals(IAnnotations(obj).get('foo'), 'bar')

    def test_simple_persistent_mapping(self):
        obj = self.create_dummy()
        alsoProvides(obj, IAttributeAnnotatable)

        data = {'foo': {'bar': 'baz'}}
        self.updater.update(obj, data)

        foo = IAnnotations(obj).get('foo')
        self.assertTrue(isinstance(foo, PersistentMapping))

    def test_simple_persistent_list(self):
        obj = self.create_dummy()
        alsoProvides(obj, IAttributeAnnotatable)

        data = {'foo': ['bar', 'baz']}
        self.updater.update(obj, data)

        foo = IAnnotations(obj).get('foo')
        self.assertTrue(isinstance(foo, PersistentList))

    def test_nested_persistent_mapping(self):
        obj = self.create_dummy()
        alsoProvides(obj, IAttributeAnnotatable)

        data = {'foo': {'bar': {'baz': 'baz'}}}
        self.updater.update(obj, data)

        foo = IAnnotations(obj).get('foo')
        bar = foo.get('bar')
        self.assertTrue(isinstance(bar, PersistentMapping))

    def test_nested_persistent_list(self):
        obj = self.create_dummy()
        alsoProvides(obj, IAttributeAnnotatable)

        data = {'foo': [['bar', 'baz']]}
        self.updater.update(obj, data)

        foo = IAnnotations(obj).get('foo')
        bar = foo[0]
        self.assertTrue(isinstance(bar, PersistentList))



# pylint: disable=E0211, E0213
# E0211: Method has no argument
# E0213: Method should have "self" as first argument

from Products.CMFPlone.factory import _DEFAULT_PROFILE
from zope.interface import Interface
from zope.schema import Bool
from zope.schema import List
from zope.schema import TextLine


# these profiles will be installed automatically
EXTENSION_PROFILES = (
    'plonetheme.classic:default',
    'plonetheme.sunburst:default',
    )


class IInflatorBundle(Interface):
    """Marker interface for inflator bundle utilties.
    """

    def __init__(title, profiles, description=u'', base=_DEFAULT_PROFILE,
                 standard=False):
        """Stores the arguments as attributes.
        """

    def install(app, site_id, title=None, language='en',
                extension_profiles=None):
        """Installs a new plone site and returns it.
        """

    title = TextLine(
        title=u'Title',
        description=u'Bundle title.')

    profiles = List(
        title=u'Profiles',
        description=u'Generic setup profile names (without profile- prefix)',
        value_type=TextLine())

    description = TextLine(
        title=u'Description',
        description=u'Optional description for the bundle.')

    base = TextLine(
        title=u'Generic Setup base profile',
        description=u'Used as base profile when creating the site.'
            u' Defaults to %s' % _DEFAULT_PROFILE)

    standard = Bool(
        title=u'Standard profile',
        description=u'Standard profiles are listed below'
        u' non-standard profiles.')


class IInflatorCustomization(Interface):
    """Adapter interface for inflator customizations adapter.
    """

    def __init__(product=None, image=None, order=10):
        """Adapts the zope application.
        """

    product = TextLine(
        title=u'Product title',
        description=u'The title of the product to be used for the wizard.')

    image = TextLine(
        title=u'The resource name of the image',
        description=u'e.g. ++resources++the-image-1.jpg')



from collective.transmogrifier.transmogrifier import Transmogrifier
import os


FOLDER_NAME = 'content_creation'


def content_creation(setup):
    """Create content based on json definitions with a folder
    "content_creation".
    """

    data = setup.isDirectory(FOLDER_NAME)
    if not data:
        return

    path = os.path.join(setup._profile_path, FOLDER_NAME).encode('utf-8')
    assert os.path.isdir(path)

    mogrifier = Transmogrifier(setup.getSite())
    mogrifier(u'ftw.inflator.creation.content_creation_config',
              jsonsource=dict(directory=path))



from Products.ATContentTypes.lib import constraintypes
from Products.CMFCore.utils import getToolByName
from ftw.inflator.testing import INFLATOR_FIXTURE
from ftw.inflator.tests.interfaces import IFoo
from persistent.list import PersistentList
from persistent.mapping import PersistentMapping
from plone.app.testing import IntegrationTesting
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import applyProfile
from plone.uuid.interfaces import IUUID
from unittest2 import TestCase
from zope.annotation.interfaces import IAnnotations


class FooCreationLayer(PloneSandboxLayer):

    defaultBases = (INFLATOR_FIXTURE, )
    def setUpPloneSite(self, portal):
        wftool = getToolByName(portal, 'portal_workflow')
        wftool.setChainForPortalTypes(['Folder'],
                                      'simple_publication_workflow')

        applyProfile(portal, 'ftw.inflator:setup-language')
        applyProfile(portal, 'ftw.inflator.tests:foo_creation')


FOO_FIXTURE = FooCreationLayer()
FOO_INTEGRATION_TESTING = IntegrationTesting(
    bases=(FOO_FIXTURE, ), name="ftw.inflator:FOO Integration")


class TestContentCreation(TestCase):

    layer = FOO_INTEGRATION_TESTING

    def setUp(self):
        super(TestContentCreation, self).setUp()
        self.portal = self.layer['portal']

    def test_foo_folder_type(self):
        foo = self.portal.get('foo')
        self.assertNotEqual(foo, None, 'Folder /foo not created')

        self.assertEqual(
            foo.portal_type, 'Folder',
            '/foo has wrong type "%s" but should be "Folder"' % (
                foo.portal_type))

    def test_foo_folder_title(self):
        foo = self.portal.get('foo')
        self.assertNotEqual(foo, None, 'Folder /foo not created')

        self.assertEqual(
            foo.Title(), 'Foo',
            '/foo has wrong Title "%s" but should be "Foo"' % foo.Title())

    def test_foo_folder_catalog(self):
        foo_path = '/'.join(self.portal.get('foo').getPhysicalPath())
        query = {'path': {'query': foo_path,
                          'depth': 0}}

        catalog = getToolByName(self.portal, 'portal_catalog')
        brains = catalog.unrestrictedSearchResults(query)
        self.assertEqual(len(brains), 1)

        foo_brain = brains[0]
        self.assertEqual(foo_brain.portal_type, 'Folder')
        self.assertEqual(foo_brain.Title, 'Foo')

    def test_foo_folder_workflow_state(self):
        foo = self.portal.get('foo')
        context_state = foo.unrestrictedTraverse('@@plone_context_state')
        review_state = context_state.workflow_state()
        self.assertEqual(review_state, 'published')

    def test_foo_folder_properties(self):
        foo = self.portal.get('foo')
        self.assertEqual(foo.getProperty('f\xc3\xbc\xc3\xbc'), 'b\xc3\xa4r')

    def test_foo_folder_constraintypes(self):
        foo = self.portal.get('foo')

        self.assertEqual(foo.getConstrainTypesMode(),
                         constraintypes.ENABLED,
                         'Constraint types are not enabled on folder foo')

        self.assertEqual(foo.getImmediatelyAddableTypes(),
                         ('Folder',),
                         'Immediately addable types are not configured well')

        self.assertEqual(foo.getLocallyAllowedTypes(),
                         ('Folder', 'Document'),
                         'Locally addable types are not configured well')

    def test_foo_folder_interfaces(self):
        foo = self.portal.get('foo')

        self.assertTrue(IFoo.providedBy(foo),
                        'IFoo interface not provided by Folder foo')

    def test_file_inserted(self):
        foo = self.portal.get('foo')
        files = foo.get('files')
        example_file = files.get('example-file')

        self.assertTrue(example_file)
        self.assertEqual(example_file.Title(), 'example file')

        data = example_file.getField('file').get(example_file)
        self.assertEqual(data.getFilename(), 'examplefile.txt')
        self.assertEqual(data.getContentType(), 'text/plain')
        self.assertEqual(data.data, 'a simple text file')

    def test_intranet_placeful_workflow(self):
        intranet = self.portal.get('foo').get('intranet')

        wftool = getToolByName(self.portal, 'portal_workflow')
        workflows = wftool.getWorkflowsFor(intranet)
        self.assertEqual(len(workflows), 1)
        wfid = workflows[0].id
        self.assertEqual(wfid, 'intranet_folder_workflow')

    def test_annotations(self):
        foo = self.portal.get('foo')
        intranet = foo.get('intranet')

        annotations = IAnnotations(intranet)
        self.assertEqual(annotations.get('foo'), {
                'bar': [1, 2, 'three'],
                'baz': True})

        self.assertFalse(isinstance({}, PersistentMapping))
        self.assertTrue(isinstance(annotations.get('foo'),
                                   PersistentMapping))

        self.assertFalse(isinstance({}, PersistentList))
        self.assertTrue(isinstance(annotations.get('foo').get('bar'),
                                   PersistentList))

    def test_intranet_translated_title(self):
        intranet = self.portal.get('foo').get('intranet')
        self.assertEqual(intranet.Title(), 'Intranet')

    def test_resolve_uuid(self):
        foo = self.portal.get('foo')
        foo_uuid = IUUID(foo)

        files = foo.get('files')
        annotations = IAnnotations(files)

        self.assertEquals(annotations.get('relative-parent-uuid'), foo_uuid)
        self.assertEquals(annotations.get('absolute-parent-uuid'), foo_uuid)

    def test_image_object(self):
        chuck = self.portal.get('foo').get('files').get('chuck-norris')
        self.assertTrue(chuck,
                        'Where is chuck norris? Image object not'
                        ' found under /foo/files/chuck-norris')

        self.assertEquals('Chuck Norris', chuck.Title())
        self.assertTrue(chuck.getSize(), 'The chuck norris image seems to be missing')
        self.assertEquals(chuck.getWidth(), 319)
        self.assertEquals(chuck.getHeight(), 397)

    def test_filename_can_be_changed(self):
        obj = self.portal.get('foo').get('files').get('filename-changed')
        self.assertEquals('filename-has-changed.jpg',
                          obj.getImage().getFilename())

    def test_news_item_object(self):
        item = self.portal.get('foo').get('in-other-news')
        self.assertTrue(item, 'Missing News Item at foo/in-other-news')

        self.assertEquals('In other news', item.Title(), 'Wrong News Item title')
        self.assertTrue(item.getText(), 'News item has no text')

    def test_local_roles_are_created(self):
        obj = self.portal.get('foo')
        roles = dict(obj.get_local_roles())
        self.assertEqual(("Owner", "Contributor", "Editor",),
                         roles.get("admin"))
        self.assertEqual(("Editor",), roles.get("hans"))

    def test_block_local_roles(self):
        obj = self.portal.get('foo')
        self.assertTrue(obj.__ac_local_roles_block__)

        intranet = obj.get('intranet')
        self.assertFalse(hasattr(intranet, '__ac_local_roles_block__'))



from Products.CMFPlone.factory import _DEFAULT_PROFILE
from Products.Five.browser.metaconfigure import resource
from ftw.inflator.bundle import InflatorBundle
from ftw.inflator.customization import InflatorCustomization
from ftw.inflator.interfaces import IInflatorBundle
from ftw.inflator.interfaces import IInflatorCustomization
from zope.component.zcml import handler
from zope.configuration.fields import Bool
from zope.configuration.fields import MessageID
from zope.configuration.fields import Path
from zope.configuration.fields import Tokens
from zope.interface import Interface
from zope.schema import Int
from zope.schema import TextLine
import os.path


class IRegisterBundleDirective(Interface):

    title = MessageID(
        title=u'Title',
        description=u'Bundle title.',
        default=None,
        required=True)

    profiles = Tokens(
        title=u'Profiles',
        description=u'Generic setup profile names (without profile- prefix)',
        default=None,
        required=True,
        value_type=TextLine())

    description = MessageID(
        title=u'Description',
        description=u'Optional description for the bundle.',
        default=u'',
        required=False)

    base = TextLine(
        title=u'Generic Setup base profile',
        description=u'Used as base profile when creating the site.'
        u' Defaults to %s' % _DEFAULT_PROFILE,
        default=_DEFAULT_PROFILE.decode('utf-8'),
        required=False)

    standard = Bool(
        title=u'Standard profile',
        description=u'Standard profiles are listed below'
        u' non-standard profiles.',
        default=False,
        required=False)


def registerBundle(_context, **kwargs):
    """Register a bundle.
    """

    component = InflatorBundle(**kwargs)
    provides = IInflatorBundle
    name = kwargs['title']

    _context.action(
        discriminator=('bundle', name),
        callable=handler,
        args=('registerUtility', component, provides, name),
        kw={'factory': None})


class IRegisterCustomizationDirective(Interface):

    product = MessageID(
        title=u'Product title',
        default=None,
        required=False)

    image = Path(
        title=u'Path to the logo image',
        default=None,
        required=False)

    order = Int(
        title=u'Customization odering number',
        default=10,
        required=False)


def registerCustomization(_context, **kwargs):
    """Register a customization.
    """
    name = str(kwargs.get('order', 10))

    if 'image' in kwargs:
        image_path = kwargs['image']
        basename, ext = os.path.splitext(os.path.basename(image_path))
        resourcename = 'inflator-%s-%s%s' % (basename, name, ext)
        resource(_context, resourcename, file=image_path)
        kwargs['image'] = '++resource++%s' % resourcename.encode('utf-8')

    component = InflatorCustomization(**kwargs)
    provides = IInflatorCustomization

    _context.action(
        discriminator=('bundle', name),
        callable=handler,
        args=('registerUtility', component, provides, name))



from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from ftw.inflator.creation.sections.resolvepath import ResolvePath
from plone.uuid.interfaces import IUUID
from zope.interface import classProvides
from zope.interface import implements


class ResolveUUID(ResolvePath):

    PREFIX = 'resolveUUID::'

    classProvides(ISectionBlueprint)
    implements(ISection)

    def _resolve(self, item, value):
        obj = super(ResolveUUID, self)._resolve(item, value)
        return IUUID(obj)



from plone.app.textfield import RichText
from plone.directives.form import Schema
from plone.formwidget.contenttree import ObjPathSourceBinder
from plone.namedfile.field import NamedImage
from z3c.relationfield.schema import RelationChoice
from zope import schema
from zope.interface import Interface


class IFoo(Interface):
    """Mark it foo!
    """


class IExampleDxType(Schema):
    image = NamedImage(
        title=u'Image',
        )

    relation = RelationChoice(
        title=u'Relation',
        source=ObjPathSourceBinder(),
        )

    date = schema.Date(
        title=u'Date',
        required=False,
    )

    datetime = schema.Datetime(
        title=u'Datetime',
        required=False,
    )

    richtext = RichText(
        title=u'Richtext',
        required=False,
        allowed_mime_types=('text/html',))



from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from collective.transmogrifier.utils import defaultMatcher
from ftw.inflator.creation.sections.helpers import map_recursive
from zope.interface import classProvides
from zope.interface import implements


class ResolvePath(object):

    PREFIX = 'resolvePath::'

    classProvides(ISectionBlueprint)
    implements(ISection)

    def __init__(self, transmogrifier, name, options, previous):
        self.transmogrifier = transmogrifier
        self.context = transmogrifier.context
        self.previous = previous
        self.pathkey = defaultMatcher(options, 'path-key', name, 'path')

    def __iter__(self):
        for item in self.previous:
            yield map_recursive(
                lambda value: value.startswith(self.PREFIX),
                lambda value: self._resolve(item, value),
                item)

    def _resolve(self, item, value):
        value = value.replace(self.PREFIX, '', 1)

        return self._get_object_by_path(item, value)

    def _get_object_by_path(self, item, path):
        if isinstance(path, unicode):
            path = path.encode('utf-8')

        path = path.rstrip('/')

        if path.startswith('/'):
            return self.context.unrestrictedTraverse(path.lstrip('/'))
        else:
            item_obj = self._get_obj(item)
            return item_obj.unrestrictedTraverse(path)

    def _get_obj(self, item):
        path = item.get(self.pathkey(*item.keys())[0], None)
        obj = self.context.unrestrictedTraverse(
            path.encode('utf-8').lstrip('/'), None)
        return obj



from ftw.inflator.testing import ZOPE_FUNCTIONAL_TESTING
from plone.app.testing import SITE_OWNER_NAME, SITE_OWNER_PASSWORD
from plone.testing.z2 import Browser
from plone.testing.z2 import zopeApp
from unittest2 import TestCase


class TestInflateView(TestCase):

    layer = ZOPE_FUNCTIONAL_TESTING

    def setUp(self):
        with zopeApp() as app:
            self.app = app

            self.browser = Browser(app)
            self.browser.handleErrors = False
            self.browser.addHeader('Authorization', 'Basic %s:%s' % (
                    SITE_OWNER_NAME, SITE_OWNER_PASSWORD))

    def test_overview_view(self):
        self.browser.open('http://localhost/index_html')
        self.browser.getControl('Create a new Plone site').click()
        self.assertEquals(self.browser.url,
                          'http://localhost/@@inflate?')



import os
from setuptools import setup, find_packages


version = '1.8.0'


extras_require = {
    'dexterity': [
        'plone.app.dexterity',
        'transmogrify.dexterity',
        ],

    'multilingual': [
        'plone.app.multilingual',
        'plone.multilingualbehavior',  # both are required
        ]}


extras_require['tests'] = tests_require = [
    'unittest2',
    'ftw.testing',
    'zope.configuration',
    'plone.testing',
    'plone.app.testing',
    'Products.CMFPlacefulWorkflow',
    ] + reduce(list.__add__, extras_require.values())


setup(name='ftw.inflator',
      version=version,
      description='Plone site setup wizard with content creation and ' + \
          'bundle system for predefined configurations.',

      long_description=open('README.rst').read() + '\n' + \
          open(os.path.join('docs', 'HISTORY.txt')).read(),

      classifiers=[
        'Framework :: Plone',
        'Framework :: Plone :: 4.2',
        'Framework :: Plone :: 4.3',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],

      keywords='ftw inflator',
      author='4teamwork AG',
      author_email='mailto:info@4teamwork.ch',
      url='https://github.com/4teamwork/ftw.inflator',

      license='GPL2',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['ftw', ],
      include_package_data=True,
      zip_safe=False,

      install_requires=[
        'setuptools',

        'zope.annotation',
        'zope.component',
        'zope.configuration',
        'zope.dottedname',
        'zope.i18n',
        'zope.i18nmessageid',
        'zope.interface',
        'zope.schema',
        'ZODB3',
        'Zope2',

        'plone.i18n',
        'plone.uuid',
        'Products.ATContentTypes',
        'Products.CMFCore',
        'Products.CMFPlone',
        'Products.GenericSetup',

        'plone.app.transmogrifier',
        'collective.transmogrifier',
        'collective.blueprint.jsonmigrator',

        'ftw.profilehook',
        ],

      tests_require=tests_require,
      extras_require=extras_require,

      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      )



from Products.CMFPlone.utils import base_hasattr
from ftw.inflator.creation.sections import constraintypes
from ftw.testing import MockTestCase


class TestConstraintypesUpdater(MockTestCase):

    def setUp(self):
        super(TestConstraintypesUpdater, self).setUp()
        transmogrifier = self.create_dummy(context=None)
        options = {'blueprint': ''}
        self.updater = constraintypes.ConstraintypesUpdater(
            transmogrifier, '', options, [])

    def test_nothing_done_when_no_constrain_support(self):
        obj = self.create_dummy()
        self.assertFalse(base_hasattr(obj, 'getConstrainTypesMode'))

    def test_set_constrain_types(self):
        obj = self.mocker.mock()
        self.expect(base_hasattr(obj, 'getConstrainTypesMode')).result(True)
        self.expect(obj.setConstrainTypesMode(1))
        self.expect(obj.setImmediatelyAddableTypes(('Folder',)))
        self.expect(obj.setLocallyAllowedTypes(('Folder', 'Page')))
        self.replay()

        data = {'locally': ['Folder', 'Page'],
                'immediately': ['Folder']}
        self.updater.update(obj, data)



from Testing.ZopeTestCase.utils import setupCoreSessions
from collective.transmogrifier import transmogrifier
from ftw.inflator.patches import apply_patches
from ftw.testing import ComponentRegistryLayer
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import SITE_OWNER_NAME, SITE_OWNER_PASSWORD
from plone.app.testing import applyProfile
from plone.app.testing.layers import PloneFixture
from plone.testing import z2
from zope.configuration import xmlconfig


def clear_transmogrifier_registry():
    # pylint: disable=W0212
    transmogrifier.configuration_registry._config_info = {}
    transmogrifier.configuration_registry._config_ids = []
    # pylint: enable=W0212


class MetaZCMLLayer(ComponentRegistryLayer):

    def setUp(self):
        super(MetaZCMLLayer, self).setUp()

        import ftw.inflator
        self.load_zcml_file('meta.zcml', ftw.inflator)


META_ZCML = MetaZCMLLayer()


class ZCMLLayer(ComponentRegistryLayer):

    def setUp(self):
        super(ZCMLLayer, self).setUp()

        import ftw.inflator.tests
        self.load_zcml_file('configure.zcml', ftw.inflator.tests)


ZCML = ZCMLLayer()


class ZopeLayer(PloneFixture):
    # we use the PloneFixture but do not create the plone site.

    defaultBases = (z2.STARTUP, )

    def setUpProducts(self, app):
        super(ZopeLayer, self).setUpProducts(app)

        configurationContext = self['configurationContext']

        # Plone < 4.3
        import Products.GenericSetup
        xmlconfig.file('meta.zcml', Products.GenericSetup,
                       context=configurationContext)

        import Products.CMFPlacefulWorkflow
        xmlconfig.file('configure.zcml', Products.CMFPlacefulWorkflow,
                       context=configurationContext)

        z2.installProduct(app, 'Products.CMFPlacefulWorkflow')

        import Products.Five
        xmlconfig.file('meta.zcml', Products.Five,
                       context=configurationContext)

        import ftw.inflator
        xmlconfig.file('configure.zcml', ftw.inflator,
                       context=configurationContext)
        xmlconfig.file('configure.zcml', ftw.inflator.tests,
                       context=configurationContext)

        apply_patches()

    def setUpDefaultContent(self, app):
        # do not create plone site
        with z2.zopeApp() as app:
            app['acl_users'].userFolderAddUser(
                    SITE_OWNER_NAME,
                    SITE_OWNER_PASSWORD,
                    ['Manager'],
                    []
                )

    def tearDown(self):
        super(ZopeLayer, self).tearDown()
        clear_transmogrifier_registry()


ZOPE_LAYER = ZopeLayer()
ZOPE_FUNCTIONAL_TESTING = z2.FunctionalTesting(
    bases=(ZOPE_LAYER, ),
    name='ftw.inflator:ZOPE_FUNCTIONAL_TESTING')


class InflatorLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, )

    def setUpZope(self, app, configurationContext):
        import Products.CMFPlacefulWorkflow
        xmlconfig.file('configure.zcml', Products.CMFPlacefulWorkflow,
                       context=configurationContext)

        z2.installProduct(app, 'Products.CMFPlacefulWorkflow')

        import ftw.inflator
        xmlconfig.file('configure.zcml', ftw.inflator,
                       context=configurationContext)

        xmlconfig.file('configure.zcml', ftw.inflator.tests,
                       context=configurationContext)

        import plone.app.multilingual
        xmlconfig.file('configure.zcml', plone.app.multilingual,
                       context=configurationContext)

        import plone.app.dexterity
        xmlconfig.file('configure.zcml', plone.app.dexterity,
                       context=configurationContext)

        setupCoreSessions(app)

    def setUpPloneSite(self, portal):
        applyProfile(
            portal, 'Products.CMFPlacefulWorkflow:CMFPlacefulWorkflow')

    def tearDown(self):
        super(InflatorLayer, self).tearDown()
        clear_transmogrifier_registry()


INFLATOR_FIXTURE = InflatorLayer()
INFLATOR_INTEGRATION_TESTING = IntegrationTesting(
    bases=(INFLATOR_FIXTURE, ), name="ftw.inflator:Integration")
INFLATOR_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(INFLATOR_FIXTURE, ), name="ftw.inflator:Functional")



from Products.CMFCore.utils import getToolByName
from collective.transmogrifier.interfaces import ISectionBlueprint
from ftw.inflator.creation.sections.base import ObjectUpdater
from zope.interface import classProvides


class PlacefulworkflowUpdater(ObjectUpdater):
    """Placeful workflow updater section, activating a placeful workflow
    policy.

    Example:
    {
        "_path": "foo/intranet",
        "_type": "Folder",
        "title": "Intranet",
        "_placefulworkflow": ["one-state", "intranet"]
    }

    This activates the placeful workflow policy "one-state" for the
    "intranet" object, but the "intranet" workflow policy for children.
    """

    classProvides(ISectionBlueprint)

    key_option_name = 'placefulworkflow-key'
    default_key_name = 'placefulworkflow'

    def update(self, obj, data):
        policy_in, policy_below = data

        placeful_workflow = getToolByName(obj, 'portal_placeful_workflow')
        config = placeful_workflow.getWorkflowPolicyConfig(obj)

        if not config:
            obj.manage_addProduct[
                'CMFPlacefulWorkflow'].manage_addWorkflowPolicyConfig()
            config = placeful_workflow.getWorkflowPolicyConfig(obj)

        config.setPolicyIn(policy=policy_in)
        config.setPolicyBelow(policy=policy_below, update_security=True)



from collective.transmogrifier.interfaces import ISection
from collective.transmogrifier.interfaces import ISectionBlueprint
from ftw.inflator.creation.sections import translate
from ftw.inflator.testing import ZCML
from ftw.testing import MockTestCase
from zope.interface.verify import verifyClass
from zope.interface.verify import verifyObject


class TestTranslateBlueprint(MockTestCase):

    layer = ZCML

    def test_implements_interface(self):
        klass = translate.Translate

        self.assertTrue(ISection.implementedBy(klass),
                        'Class %s does not implement ISection.' % str(klass))
        verifyClass(ISection, klass)

        self.assertTrue(ISectionBlueprint.providedBy(klass),
                        'Class %s does not provide ISectionBlueprint.' % (
                str(klass)))
        verifyObject(ISectionBlueprint, klass)

    def test_translates(self):
        transmogrifier = self.stub()
        self.expect(transmogrifier.context.Language()).result('en')
        self.replay()

        input = [
            {'_id:translate(ftw.inflator.tests)': 'intranet-id',
             'title:translate(ftw.inflator.tests)': 'intranet-title',
             'foo': [{'title:translate(ftw.inflator.tests)': 'intranet-title'}]}]

        expected = [
            {'_id': u'intranet',
             'title': u'Intranet',
             'foo': [{'title': u'Intranet'}]}]

        source = translate.Translate(transmogrifier, '', None, input)
        output = list(source)

        self.maxDiff = None
        self.assertEqual(output, expected)

    def test_translates_encoding(self):
        transmogrifier = self.stub()
        self.expect(transmogrifier.context.Language()).result('en')
        self.replay()

        # UTF-8 encoded value for the title.
        input = [
            {'_id:translate(ftw.inflator.tests)': 'burger',
             'title:translate(ftw.inflator.tests)': 'B\xc3\xbcrger'}]

        expected = [
            {'_id': u'burger',
             'title': u'B\xfcrger'}]

        source = translate.Translate(transmogrifier, '', None, input)
        output = list(source)

        self.maxDiff = None
        self.assertEqual(output, expected)



from OFS.ObjectManager import ObjectManager
from ftw.inflator.customization import get_merged_customizations
from logging import getLogger


LOG = getLogger('ftw.inflator.patches')


def apply_patches():
    patch_ZMI_button()


def patch_ZMI_button():
    main = ObjectManager.manage_main
    orig = main.read()

    if 'ftw.inflator action' in orig:
        return False

    LOG.info('Patch install product button into ZMI.')

    main.inflator_customization = get_merged_customizations

    ADD_PLONE_SITE_HTML = '''
    <dtml-if "_.len(this().getPhysicalPath()) == 1 or this().meta_type == 'Folder' and 'PloneSite' not in [o.__class__.__name__ for o in this().aq_chain]">
      <!-- ftw.inflator action -->
      <form method="get"
            action="&dtml-URL1;/@@inflate"
            style="text-align: right; margin-top:0.5em; margin-bottom:0em;"
            target="_top">
        <input type="submit" value='Install <dtml-var expr="this().manage_main.inflator_customization().product">' style="font-size: 140%" />
      </form>
    </dtml-if>
    '''

    pos = orig.find('<!-- Add object widget -->')

    # Add in our button html at the right position
    new = orig[:pos] + ADD_PLONE_SITE_HTML + orig[pos:]

    # Modify the manage_main
    main.edited_source = new
    main._v_cooked = main.cook()

    return True

apply_patches()



from Products.CMFPlone.factory import _DEFAULT_PROFILE
from Products.CMFPlone.factory import addPloneSite
from ftw.inflator.interfaces import EXTENSION_PROFILES
from ftw.inflator.interfaces import IInflatorBundle
from zope.component import getUtilitiesFor
from zope.interface import implements


class InflatorBundle(object):
    """Inflator bundle utility with information about the bundle.
    """

    implements(IInflatorBundle)

    def __init__(self, title, profiles, description=u'', base=_DEFAULT_PROFILE,
                 standard=False):
        self.title = title
        self.profiles = profiles
        self.description = description
        self.base = base
        self.standard = standard

    def install(self, app, site_id, title=None, language='en',
                extension_profiles=EXTENSION_PROFILES):

        if title is None:
            title = self.title

        extension_profiles = list(extension_profiles)
        extension_profiles.extend(self.profiles)

        return addPloneSite(
            app,
            site_id,
            title=title,
            profile_id=self.base,
            extension_ids=extension_profiles,
            setup_content=False,
            default_language=language)


def get_bundles():
    """Returns a list of all registered bundles.
    """

    def sort_key(item):
        name, bundle = item
        if bundle.standard:
            return 1, name
        else:
            return 0, name

    bundles = sorted(getUtilitiesFor(IInflatorBundle),
                     key=sort_key)

    return [bundle for (_name, bundle) in bundles]


def get_bundle_by_name(name):
    """Returns the bundle with the name `name`.
    """
    matches = filter(lambda bundle: bundle.title == name,
                     get_bundles())

    if len(matches) == 0:
        raise ValueError('No bundle found with name %s' % name)
    elif len(matches) > 1:
        raise ValueError('Too many bundles found with name %s' % name)

    return matches[0]



from Products.CMFPlone.factory import _DEFAULT_PROFILE
from ftw.inflator.bundle import InflatorBundle
from ftw.inflator.interfaces import IInflatorBundle
from unittest2 import TestCase
from zope.interface.verify import verifyClass


class TestBundleUtility(TestCase):

    def test_bundle_implements_interface(self):
        self.assertTrue(IInflatorBundle.implementedBy(InflatorBundle))
        verifyClass(IInflatorBundle, InflatorBundle)

    def test_stores_arguments(self):
        obj = InflatorBundle(title=u'foo',
                             profiles=[u'bar'],
                             description=u'baz',
                             base=u'foo:default',
                             standard=True)

        self.assertEqual(obj.title, u'foo')
        self.assertEqual(obj.profiles, [u'bar'])
        self.assertEqual(obj.description, u'baz')
        self.assertEqual(obj.base, u'foo:default')
        self.assertEqual(obj.standard, True)

    def test_defaults(self):
        obj = InflatorBundle(title=u'foo', profiles=[u'bar'])

        self.assertEqual(obj.title, u'foo')
        self.assertEqual(obj.profiles, [u'bar'])
        self.assertEqual(obj.description, u'')
        self.assertEqual(obj.base, _DEFAULT_PROFILE)
        self.assertEqual(obj.standard, False)

    def test_requires(self):
        with self.assertRaises(TypeError) as cm:
            InflatorBundle()

        self.assertEquals(str(cm.exception),
                          '__init__() takes at least 3 arguments (1 given)')
